/* 
 * FPV Arduino RC Tank Code   
 */

//Libraries to include
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include <Wire.h>
#include "PCA9685.h"

//Intitializtion & Declaration & Definition  
RF24 radio (8, 5); //CE, CSN
const byte address[6] = "00002";

typedef struct { //declare stucture
  int Lxj; //members
  int Lyj;
  int Rxj;
  int Ryj;
  int PotL;
  int PotR;
  int TS1;
  int TS2;
}Data;
Data value; //value is object

int ackData = 0;
unsigned long pT = 0; //Previous Time
const long interval = 1000; //For after a second the controller turn off the Tank will Stop

//Setting up the PCA9685 servo driver
PCA9685 driver;
int ESCL, ESCR, diff; // values for throttling Left and Right brushless motor
//// int traverse; // values for turret Traverse servo motor position
//// int elevation; //values for gun elevation servo motor position 

// PCA9685 outputs = 12-bit = 4096 steps
// 1ms to 2ms is the PWM signal for ESC 
// 1/50Hz = 0.02 second = 20ms ; 1ms/20ms = 0.05 ; 2ms/20ms = 0.1  
// 4096*0.05 = 205 steps ; 4096*0.1 = 410 steps 
PCA9685_ServoEval pwmServoB(200, 409); // (-90deg, +90deg)
// 0.5ms to 2.5ms is the PWM signal for MG90S servo motor () 
// 2.5% of 20ms = 0.5ms ; 12.5% of 20ms = 2.5ms
// 2.5% of 4096 = 102 steps; 12.5% of 4096 = 512 steps
PCA9685_ServoEval pwmServoS(102, 512); // (-90deg, +90deg)

const int Relay1 = 2; //The relay signal pins that connect to the Arduino 
const int RelayL1 = 10;
const int RelayL2 = 4;
const int RelayR1 = 6;
const int RelayR2 = 7;

bool i = false; //For selecting Forward and Backward
int TA = 0;     //Traverse angle
int EA = 0;     //Elevation angle

void setup() {
  Serial.begin(9600);
  
  radio.begin();
  radio.enableAckPayload();
  radio.openReadingPipe(1, address);
  radio.setPALevel(RF24_PA_MIN);
  radio.startListening();

  Wire.begin();                 // Wire must be started first
  Wire.setClock(400000);        // Supported baud rates are 100kHz, 400kHz, and 1000kHz
  driver.resetDevices();        // Software resets all PCA9685 devices on Wire line
  driver.init();                // Address pins A5-A0 set to B000000
  driver.setPWMFrequency(50);   // Set frequency to 50Hz
  
  pinMode(Relay1, OUTPUT);  //Single channel relay pin
  pinMode(RelayL1, OUTPUT); //Left double Channel relay pin IN1
  pinMode(RelayL2, OUTPUT); //Left double Channel relay pin IN2
  pinMode(RelayR1, OUTPUT); //Right double Channel relay pin IN1
  pinMode(RelayR2, OUTPUT); //Right double Channel relay pin IN2 
  
  driver.setChannelPWM(0, pwmServoS.pwmForAngle(0));
  driver.setChannelPWM(1, pwmServoS.pwmForAngle(0));
  driver.setChannelPWM(7, pwmServoB.pwmForAngle(-90));
  driver.setChannelPWM(8, pwmServoB.pwmForAngle(-90));
  digitalWrite(Relay1, LOW); //Setup all the relays to Normally Open positions
  digitalWrite(RelayL1, HIGH);
  digitalWrite(RelayL2, HIGH);
  digitalWrite(RelayR1, HIGH);
  digitalWrite(RelayR2, HIGH);
  Serial.print("Start");
  delay(5000);
}


void loop() {
  unsigned long cT = millis(); //Current Time
  
  //Receive Data from controller 
  if ( radio.available() ){
    radio.read( &value, sizeof(value) );
    ackData += 1;
    if (ackData >= 10) {
      ackData = 0;
    }

    radio.writeAckPayload(1, &ackData, sizeof(ackData)); //Sending data back to the controller for confirming that data has been recived 
    Serial.print(" Data Count ");
    Serial.print(ackData);
    delay(50);

    pT = cT; //make previous time equal current time so it recounts

    //Print values for troubleshooting
    Serial.print(" ON ");
    Serial.print("  PotL: ");
    Serial.print(value.PotL);
    Serial.print("  PotR: ");
    Serial.print(value.PotR);
    Serial.print("  TS1: ");
    Serial.print(value.TS1);
    Serial.print("  i: ");
    Serial.print(i);
    Serial.print("  ESCL: ");
    Serial.print(ESCL);
    Serial.print("  ESCR: ");
    Serial.print(ESCR);
    Serial.print(" diff: ");
    Serial.print(diff);
    Serial.print(" TA: ");
    Serial.print(TA);
    Serial.print(" EA: ");
    Serial.println(EA);
    
  }
  else { // When controller not sending data it stops the car
    if (cT - pT >= interval) {
      pT = cT;
      value.TS1 = 321;
      driver.setChannelPWM(7, pwmServoB.pwmForAngle(-90));
      driver.setChannelPWM(8, pwmServoB.pwmForAngle(-90));
      digitalWrite(Relay1, LOW); //Set all the relays to Normally Open positions
      digitalWrite(RelayL1, HIGH);
      digitalWrite(RelayL2, HIGH);
      digitalWrite(RelayR1, HIGH);
      digitalWrite(RelayR2, HIGH);
      Serial.println(" No Signal ");
      i = false;
      delay(500);
    }
  }

  if (value.TS1 == 1){ //GO forward
    if (i == false){
      //Startup Sequence
      driver.setChannelPWM(7, pwmServoB.pwmForAngle(-90));
      driver.setChannelPWM(8, pwmServoB.pwmForAngle(-90));
      //Setup relays to GO forward
      Serial.print(" Drive ");
      digitalWrite(Relay1, LOW); 
      delay(500);
      digitalWrite(RelayL1, HIGH);
      digitalWrite(RelayL2, HIGH);
      digitalWrite(RelayR1, HIGH);
      digitalWrite(RelayR2, HIGH);
      i = true;
      delay(1000);
      digitalWrite(Relay1, HIGH);
      delay(6000);
    }  

    if (value.PotL > 0){
      ESCL = map(value.PotL, 0, 1023, -90, 90);
      ESCR = map(value.PotL, 0, 1023, -90, 90);

    }
    else{
      ESCL = -90;
      ESCR = -90;
      diff = 0;
    }
    
    if (value.PotR < 500){ //Turn Left 
      diff = map(value.PotR, 500, 0, 0, 180);
      ESCL = ESCL - diff;
      ESCR = ESCR + diff; //Speed the right motor to turn left 

      if (ESCR > 90){
        ESCR = 90;
      }
      if (ESCL < -90){
        ESCL = -90; 
      }
    }
    
    if (value.PotR > 530){ //Turn Right 
      diff = map(value.PotR, 530, 1023, 0, 180);
      ESCL = ESCL + diff;
      ESCR = ESCR - diff; //Slow down the right motor to turn right 

      if (ESCL > 90){
        ESCL = 90;
      }
      if (ESCR < -90){
        ESCR = -90; 
      }
    }
    
    //Sending values to the ESCs
    driver.setChannelPWM(7, pwmServoB.pwmForAngle(ESCL));
    driver.setChannelPWM(8, pwmServoB.pwmForAngle(ESCR));   
  }

  else if (value.TS1 == 0){ //GO backward
    if (i == true){
      //Startup Sequence
      driver.setChannelPWM(7, pwmServoB.pwmForAngle(-90));
      driver.setChannelPWM(8, pwmServoB.pwmForAngle(-90));
      //Setup relays to GO backward
      Serial.print(" Reverse ");
      digitalWrite(Relay1, LOW); 
      delay(500);
      digitalWrite(RelayL1, LOW);
      digitalWrite(RelayL2, LOW);
      digitalWrite(RelayR1, LOW);
      digitalWrite(RelayR2, LOW);
      i = false;
      delay(1000);
      digitalWrite(Relay1, HIGH);
      delay(6000);
    }
    if (value.PotL > 0){
      ESCL = map(value.PotL, 0, 1023, -90, 90);
      ESCR = map(value.PotL, 0, 1023, -90, 90);
      
    }
    else{
      ESCL = -90;
      ESCR = -90;
      diff = 0;
    }

    if (value.PotR < 500){ //Turn Left 
      diff = map(value.PotR, 500, 0, 0, 180);
      ESCL = ESCL - diff;
      ESCR = ESCR + diff; //Speed the right motor to turn left  

      if (ESCR > 90){
        ESCR = 90;
      }
      if (ESCL < -90){
        ESCL = -90; 
      }
    }
    
    if (value.PotR > 530){ //Turn Right 
      diff = map(value.PotR, 530, 1023, 0, 180);
      ESCL = ESCL + diff;
      ESCR = ESCR - diff; //Slow down the right motor to turn right 

      if (ESCL > 90){
        ESCL = 90;
      }
      if (ESCR < -90){
        ESCR = -90; 
      }
    }
    
    //Sending values to the ESCs
    driver.setChannelPWM(7, pwmServoB.pwmForAngle(ESCL));
    driver.setChannelPWM(8, pwmServoB.pwmForAngle(ESCR));
  }
  else {
    driver.setChannelPWM(7, pwmServoB.pwmForAngle(-90));
    driver.setChannelPWM(8, pwmServoB.pwmForAngle(-90));
  }

  //Servo motor control for the turret
  TA = map(value.Ryj, 0, 1023, 90, -90); //Traverse angle 
  EA = map(value.Rxj, 0, 1023, -5, 40);  //Elevation angle
  
  driver.setChannelPWM(0, pwmServoS.pwmForAngle(TA));
  driver.setChannelPWM(1, pwmServoS.pwmForAngle(EA));
  
}
